'==============================================================================
'
'  Address Book for PowerBASIC for Windows PB/WIN 7.0
'  Copyright (c) 1999-2002 PowerBASIC, Inc.
'
'  A simple address book example using Dynamic Dialog Tools (DDT)
'
'  Modified by Hepika Solutions for operate with
'  PLSAR 3.0 Database Manager (Freeware) and Plsar Packed Functions
'
'  Plsar 3.0 Database Manager Freeware an Registered Versions
'  (C) 2002 by Ricardo Ponce
'  http://www.hepika.com.ar
'  rponce@lanet.com.ar
'
'  This program show how to use the Plsar engine in practical form.
'  In the begining, create a small database and save four record,
'  allowing modifiy, search delete and add new records into database.
'  The example database is created only if not exist prevously, in this
'  case, the program not create it.
'
'==============================================================================


'------------------------------------------------------------------------------
' Metastatements
'------------------------------------------------------------------------------
#COMPILE EXE
#DIM ALL


'------------------------------------------------------------------------------
' Include files
'------------------------------------------------------------------------------
#INCLUDE "WIN32API.INC"
#INCLUDE "PULSAR30LITE.INC"     'include plsar packed functions

'------------------------------------------------------------------------------
'Equates
'------------------------------------------------------------------------------
$AppTitle = "Address Book with Plsar 3.0 Lite - PB/WIN 7.0 Example"

'Edit Control IDs
%ID_INDEX       = 100
%ID_COMPANY     = 101
%ID_LASTNAME    = 102
%ID_FIRSTNAME   = 103
%ID_ADDRESS1    = 104
%ID_ADDRESS2    = 105
%ID_ADDRESS3    = 106
%ID_CITYNAME    = 107
%ID_STATENAME   = 108
%ID_ZIPCODE     = 109
%ID_COUNTRY     = 110
%ID_PHONE       = 111
%ID_FAX         = 112
%ID_EMAIL       = 113
%ID_URL         = 114
%ID_COMMENTS    = 115

'Button control IDs
%ID_PRIOR       = 121
%ID_NEXT        = 122
%ID_ADD         = 123
%ID_UPDATE      = 124
%ID_FIND        = 125
%ID_DELETE      = 126

'Menu control IDs
%ID_MENU_CLOSE  = %IDCANCEL
%ID_MENU_CUT    = 201
%ID_MENU_COPY   = 202
%ID_MENU_PASTE  = 203
%ID_MENU_HELP   = 204
%ID_MENU_ABOUT  = 205

%NO  = 0            '%YES / %NO flags are used by PriorNextCallBack function
%YES = 1            'to determine if record must be read from database

'------------------------------------------------------------------------------
'Data structures
'------------------------------------------------------------------------------
TYPE AddressType BYTE           'this structure allow save records in
    Company   AS STRING *   64  'our database
    LastName  AS STRING *   32  'is used by DatabaseRecord global variable
    FirstName AS STRING *   32
    Address1  AS STRING *   64
    Address2  AS STRING *   64
    Address3  AS STRING *   64
    CityName  AS STRING *   24
    StateName AS STRING *    4
    ZipCode   AS STRING *   12
    Country   AS STRING *   20
    Phone     AS STRING *   24
    Fax       AS STRING *   24
    Email     AS STRING *   64
    Url       AS STRING *   64
    Comments  AS STRING *  999  'field only must contains until 999 chars
END TYPE


'------------------------------------------------------------------------------
' Global data variables
'------------------------------------------------------------------------------
GLOBAL gIsRecModified  AS LONG
GLOBAL gIsFileModified AS LONG
GLOBAL gCurrentEntry   AS LONG
GLOBAL gMaxEntries     AS LONG
GLOBAL gszFindText     AS ASCIIZ * 1024

GLOBAL hDlg    AS DWORD                 'identify dialog form
GLOBAL PulsarPath$                      'used by Plsars packed functions
GLOBAL PulsarFile$                      'used by Plsars packed functions
GLOBAL DatabaseRecord AS AddressType    'used in database by operate records
GLOBAL ActualRecord$                    'actual record readed in database
GLOBAL MaximumRecord$                   'last record saved in database
'------------------------------------------------------------------------------
' Callback Declarations
'------------------------------------------------------------------------------
DECLARE CALLBACK FUNCTION AddressCallBack()
DECLARE CALLBACK FUNCTION PriorNextCallBack()
DECLARE CALLBACK FUNCTION AddUpdateCallBack()
DECLARE CALLBACK FUNCTION DeleteCallBack()
DECLARE CALLBACK FUNCTION OkCallBack()
DECLARE CALLBACK FUNCTION FindCallBack()
DECLARE CALLBACK FUNCTION FindDialogCallBack()
DECLARE CALLBACK FUNCTION CancelCallBack()

'------------------------------------------------------------------------------
' Helper Sub/Function Declarations
'------------------------------------------------------------------------------
DECLARE SUB SetButtonStatus()
DECLARE SUB AddCountries()
DECLARE FUNCTION GetDriveActual(Disco AS STRING,Directorio AS STRING) AS STRING
DECLARE SUB PresentRecord ()          'see description in subroutine
DECLARE SUB CleanScreen ()            'see description in subroutine
DECLARE SUB EnableScreen ()           'see description in subroutine
DECLARE SUB DisableScreen ()          'see description in subroutine
DECLARE SUB AssignControlTextToFields 'see description in subroutine
DECLARE SUB SaveRecordsInDataBase     'see description in subroutine

'------------------------------------------------------------------------------
' Main entry point for the application
'------------------------------------------------------------------------------
FUNCTION PBMAIN () AS LONG

    REGISTER x       AS LONG

    LOCAL    hMenu   AS DWORD
    LOCAL    Result  AS LONG
    LOCAL    hPopup1 AS DWORD
    LOCAL    hPopup2 AS DWORD
    LOCAL    nFile   AS LONG
    LOCAL    Auxil$,ActualDisk$,Engine$     'auxiliars variables
    LOCAL    SBstructure$                   'structure used in database creation
    LOCAL    A&


    '--------------------------------------------------------
    'Initialize variables
    '
    'PulsarFile$ contain the database file name
    'PulsarPath$ contain the file path
    '
    'Both variables are used in all programs routines
    'that use plsar packed functions

    PulsarFile$="ExampleDB"                         'DataBase file name
    Auxil$=GetDriveActual$(ActualDisk$,PulsarPath$) 'Get the actual path and disk
    PulsarPath$=ActualDisk$+PulsarPath$             'Used by packed functions

    '--------------------------------------------------------
    'verify plsar engine

    Engine$=PulsarEngine$()
    IF Engine$<>"3.0L" THEN
       MSGBOX "This program require Plsar 3.0 Lite installed in you system."
       EXIT FUNCTION
    END IF

    '--------------------------------------------------------
    'view pulsar screen
    'this command operate directly into plsar heart function
    A&=PULSAR(2,"","","","","")

    '--------------------------------------------------------
    'create new database if not exist previously
    '
    'the database structure is defined into SBstructure$ variable
    'according to AddressType estructure (UDT)

    SBstructure$="Company:64,LastName:32,FirstName:32,AddressA:64,AddressB:64,"
    SBstructure$=SBstructure$+"AddressC:64,CityName:24,StateName:4,ZipCode:12,Country:20,"
    SBstructure$=SBstructure$+"Phone:24,Fax:24,Email:64,Url:64,Comments:1024"

    'this plsar function only create database if not exist previously
    'if database is create, function return "ok"
    'if database exist and is not create, return "ERROR CODE:xxx...." (see plsar manual)
    Auxil$=NewConventionalDB$(SBstructure$,"Company",PulsarPath$,PulsarFile$)

    '--------------------------------------------------------
    'save records in database only if database is empty
    MaximumRecord$=GetQuantityOfRecords$(PulsarPath$,PulsarFile$)
    IF VAL(MaximumRecord$)<1 THEN
       CALL SaveRecordsInDataBase
       MaximumRecord$=GetQuantityOfRecords$(PulsarPath$,PulsarFile$)
    END IF
    ActualRecord$="1"

    '--------------------------------------------------------
    'program interface GUI

    'create primary GUI
    DIALOG NEW 0,$AppTitle,77,22,400,267,%DS_CENTER OR %WS_CAPTION OR %WS_SYSMENU,0 TO hDlg

    'abort if the dialog could not be created
    IF hDlg = 0 THEN EXIT FUNCTION  ' Error occurred
    DIALOG SET COLOR hDlg, -1&, RGB(255,255,222)

    'now we create our controls.  We create the static control immediately before the
    'real control so that the keyboard accelerators switch focus to the real control
    CONTROL ADD LABEL,    hDlg, 20,        "Record N", 303,   5,  60,   8, %SS_RIGHT
    CONTROL ADD LABEL,    hDlg, %id_Index,     "0 / 0", 366,   4,  25,  10, %SS_SUNKEN OR %SS_CENTER
    CONTROL ADD LABEL,    hDlg, 21,         "Compan&y",   5,  19,  60,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_Company,     "101",  68,  17, 323,  12
    CONTROL ADD LABEL,    hDlg, 22,       "Last Na&me",   5,  33,  60,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_LastName,    "102",  68,  31, 150,  12
    CONTROL ADD LABEL,    hDlg, 23,           "Fi&rst", 219,  33,  18,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_FirstName,   "103", 241,  31, 150,  12
    CONTROL ADD LABEL,    hDlg, 24,         "Addre&ss",   5,  48,  60,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_Address1,    "104",  68,  46, 323,  12
    CONTROL ADD TEXTBOX,  hDlg, %id_Address2,    "105",  68,  61, 323,  12
    CONTROL ADD TEXTBOX,  hDlg, %id_Address3,    "106",  68,  76, 323,  12
    CONTROL ADD LABEL,    hDlg, 25,            "Ci&ty",   5,  93,  60,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_CityName,    "107",  68,  91, 116,  12
    CONTROL ADD LABEL,    hDlg, 26,  "State/Pro&vince", 188,  93,  50,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_StateName,   "108", 241,  91,  45,  12
    CONTROL ADD LABEL,    hDlg, 27, "&Zip or Postal Code",5, 109,  60,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_ZipCode,     "109",  68, 107,  40,  12
    CONTROL ADD LABEL,    hDlg, 28,         "Co&untry", 130, 109,  25,   8, %SS_RIGHT
    CONTROL ADD COMBOBOX, hDlg, %id_Country,     , 158, 107, 129, 100, %CBS_DROPDOWNLIST OR %WS_VSCROLL OR %WS_TABSTOP
    CONTROL ADD LABEL,    hDlg, 29,           "Ph&one",   5, 127,  60,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_Phone,       "111",  68, 125, 150,  12
    CONTROL ADD LABEL,    hDlg, 30,             "Fa&x", 219, 127,  18,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_Fax,         "112", 241, 125, 150,  12
    CONTROL ADD LABEL,    hDlg, 31,           "Ema&il",   5, 144,  60,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_Email,       "113",  68, 142, 150,  12
    CONTROL ADD LABEL,    hDlg, 32,             "&www", 219, 144,  18,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_Url,         "114", 241, 142, 150,  12
    CONTROL ADD LABEL,    hDlg, 33,        "C&omments",   5, 160,  60,   8, %SS_RIGHT
    CONTROL ADD TEXTBOX,  hDlg, %id_Comments,    "115",  68, 158, 323,  65, %ES_AUTOHSCROLL OR %ES_AUTOVSCROLL OR %ES_MULTILINE OR %ES_WANTRETURN OR %WS_TABSTOP OR %WS_VSCROLL, %WS_EX_CLIENTEDGE

    'color the LABEL controls (foreground = default, background = transparent)
    FOR x = 20 TO 33
        CONTROL SET COLOR hDlg, x, -1, -2
    NEXT x

    'add the command buttons
    CONTROL ADD BUTTON,   hDlg, %id_Prior, "&Prior <<",  10, 233,  40,  14 CALL PriorNextCallBack
    CONTROL ADD BUTTON,   hDlg, %id_Next,   ">> &Next",  60, 233,  40,  14 CALL PriorNextCallBack
    CONTROL ADD BUTTON,   hDlg, %id_Add,        "&Add", 110, 233,  40,  14 CALL AddUpdateCallBack
    CONTROL ADD BUTTON,   hDlg, %id_Update,  "U&pdate", 160, 233,  40,  14 CALL AddUpdateCallBack
    CONTROL ADD BUTTON,   hDlg, %id_Find,      "&Find", 210, 233,  40,  14 CALL FindCallBack
    CONTROL ADD BUTTON,   hDlg, %id_Delete,  "&Delete", 260, 233,  40,  14 CALL DeleteCallBack
    CONTROL ADD BUTTON,   hDlg, %IDCANCEL,    "&Close", 350, 233,  40,  14 CALL CancelCallBack

    '--------------------------------------------------------
    'read the first record and present it in screen
    Auxil$=GetRecordConvDB$(PulsarPath$,PulsarFile$,ActualRecord$) 'read the record
    POKE$ VARPTR(DatabaseRecord),Auxil$  'assign record readed to User Defined Type (DataBaseRecord)
    CALL PresentRecord          'present record in screen
    CALL DisableScreen          'inactivate screen controls (no modify state)

    'start up our GUI, and run until the user quits
    DIALOG SHOW MODAL hDlg, CALL AddressCallBack TO Result

END FUNCTION


'------------------------------------------------------------------------------
' Main dialog callback
'------------------------------------------------------------------------------
CALLBACK FUNCTION AddressCallBack

    LOCAL Dummy AS LONG

    SELECT CASE CBMSG

    CASE %WM_COMMAND
        ' If any control is edited, we need to register this so the ADD and
        ' UPDATE buttons can be enabled.
        IF CBCTLMSG = %EN_UPDATE _       ' *Any* edit control was changed
        OR (CBCTL = %id_Country AND CBCTLMSG = %CBN_SELCHANGE) THEN ' The Country combobox was changed
            ' If this is the first "change" message, we need to update the
            ' controls in the dialog box.
            IF ISFALSE gIsRecModified THEN
                gIsRecModified = %TRUE
            END IF
        END IF

        ' Now process any other %WM_COMMAND message from controls that don't
        ' have their own callback functions.
        SELECT CASE CBCTL

        CASE %id_Menu_Help
            Dummy = SHELL("WINHELP.EXE", 1)

        CASE %id_Menu_About
            ' About dialog
            DIALOG NEW CBHNDL, $AppTitle, 0, 0, 170, 60, _
                        %DS_MODALFRAME OR %WS_CAPTION OR %WS_SYSMENU OR %DS_CENTER TO hDlg
            DIALOG SET COLOR hDlg, -1&, %BLUE
            IF hDlg = 0 THEN EXIT FUNCTION  ' Error occurred
            CONTROL ADD LABEL, hDlg, -1, "A PB/WIN 7.0 DDT example" & $CRLF & $CRLF & "by PowerBASIC, Inc.", _
                        15, 10, 130, 34, %SS_CENTER
            CONTROL SET COLOR hDlg, -1, %WHITE, %BLUE
            CONTROL ADD BUTTON, hDlg, 1, "&Ok", 60, 42, 40, 14 CALL OkCallBack
            BEEP
            DIALOG SHOW MODAL hDlg
        END SELECT

    END SELECT

END FUNCTION


'------------------------------------------------------------------------------
' PRIOR/NEXT Button callback function
'------------------------------------------------------------------------------
CALLBACK FUNCTION PriorNextCallBack

    LOCAL RDactual&,RDmaximun&,ReadRecord%,Auxil$,CompanyName$

    RDactual&  = VAL(ActualRecord$)
    RDmaximun& = VAL(MaximumRecord$)

    ReadRecord%=%NO          'this variable determine that the actual
                             'record are readed from database or not,
                             'default %NO state is setted

    'Activate "no modify" state
    'By default, the program not allow modify the record fields
    CALL DisableScreen  'inactivate screen controls (no modify state)
    CONTROL SET TEXT hDlg,%id_Add,"&Add"       'assure that this buttons operate
    CONTROL SET TEXT hDlg,%id_Update,"U&pdate" 'how add and update functions


    'if fields screen are empty, read the actual record
    'and present it
    CONTROL GET TEXT CBHNDL,%id_LastName TO CompanyName
    IF LEN(TRIM$(CompanyName)) = 0 THEN
       'read record and present in screen
       Auxil$=GetRecordConvDB$(PulsarPath$,PulsarFile$,ActualRecord$) 'read the record
       POKE$ VARPTR(DatabaseRecord),Auxil$  'assign record readed to UDT
                                            '(User Defined Type - DataBaseRecord)
       CALL PresentRecord          'present record in screen
       CALL DisableScreen          'inactivate screen controls (no modify state)
    END IF

    'Determine which control is originating the message
    IF CBCTL = %id_Prior THEN       '"Prior" control
        IF (RDactual&-1)>0 THEN
           RDactual&=RDactual&-1
           ActualRecord$=TRIM$(STR$(RDactual&))
           ReadRecord%=%YES
        END IF
    ELSE                            '"Next" control
        IF (RDactual&+1)<=RDmaximun& THEN
           RDactual&=RDactual&+1
           ActualRecord$=TRIM$(STR$(RDactual&))
           ReadRecord%=%YES
        END IF
    END IF


    'if ActualRecord$ variable are modified by "Prior" or "Next"
    'routines, the screen must be actualized and the new record
    'must be presented in screen

    IF ReadRecord%=%YES THEN
       'read record and present it
       Auxil$=GetRecordConvDB$(PulsarPath$,PulsarFile$,ActualRecord$) 'read the record
       POKE$ VARPTR(DatabaseRecord),Auxil$  'assign record readed to User Defined Type (DataBaseRecord)
       CALL PresentRecord          'present record in screen
       CALL DisableScreen          'inactivate screen controls (no modify state)

    END IF

END FUNCTION


'------------------------------------------------------------------------------
' ADD and UPDATE Buttons callback function
'------------------------------------------------------------------------------
CALLBACK FUNCTION AddUpdateCallBack

    LOCAL ActualStatus$ 'actual status in selected button
    LOCAL Auxil$,CompanyName$,RecordToSave$,Temp$

    'The operativity of this function is determined by
    'the texts contained into the Add and Update
    'buttons.
    '
    'This buttons are modified dinamically during the
    'programs operations.

    'The ADD button must be determine three actions:
    '       1) Add new record into database
    '       2) Cancel the actual operation
    '       3) Save the new record into database

    'The UPDATE button must be determine three actions:
    '       1) Modify the record presented in screen
    '       2) Cancel the actual operation
    '       3) Save the record modifications into database

    'Now determine if ADD or UPDATE buttons clicked

    IF CBCTL = %id_Add THEN 'The ADD button is clicked

       CONTROL GET TEXT hDlg, %id_Add TO ActualStatus$
       ActualStatus$=LCASE$(TRIM$(ActualStatus$))

       SELECT CASE ActualStatus$
          CASE "&add"    'add
              CONTROL SET TEXT hDlg,%id_Add,"SAVE"
              CONTROL SET TEXT hDlg,%id_Update,"CANCEL"
              CALL CleanScreen
              CONTROL SET TEXT hDlg,%id_Index,TRIM$(STR$(VAL(MaximumRecord$)+1))+"/"+TRIM$(STR$(VAL(MaximumRecord$)+1))
              CALL EnableScreen   'enable screen to add new record
              CALL AddCountries   'initialize our combobox Country List
              CONTROL SET FOCUS hDlg,%id_Company

          CASE "cancel"
              CONTROL SET TEXT hDlg,%id_Add,"&Add"
              CONTROL SET TEXT hDlg,%id_Update,"U&pdate"

              'read the actual record and present it
              Auxil$=GetRecordConvDB$(PulsarPath$,PulsarFile$,ActualRecord$) 'read the record
              POKE$ VARPTR(DatabaseRecord),Auxil$  'assign record readed to User Defined Type (DataBaseRecord)
              CALL PresentRecord          'present record in screen
              CALL DisableScreen          'inactivate screen controls (no modify state)

          CASE ELSE      'save the new record

              'verify that the company field are not empty because
              'the company field is the database index field
              CONTROL GET TEXT hDlg,%id_Company TO CompanyName

              IF LEN(TRIM$(CompanyName)) = 0 THEN
                 BEEP           ' Display error message dialog
                 MSGBOX "You must specify a Company Name!",%MB_ICONERROR,"PLSAR 3.0 Lite"
              ELSE
                 CONTROL SET TEXT hDlg,%id_Add,"&Add"
                 CONTROL SET TEXT hDlg,%id_Update,"U&pdate"

                 CALL AssignControlTextToFields

                 RecordToSave$=DatabaseRecord
                 Auxil$=SaveIndexConvDB(PulsarPath$,PulsarFile$,RecordToSave$)
                 IF Auxil$<>"ok" THEN
                    MSGBOX "ERROR SAVING RECORD IN DATABASE!",%MB_ICONERROR,"PLSAR 3.0 Lite"
                 END IF

                 MaximumRecord$=GetQuantityOfRecords$(PulsarPath$,PulsarFile$)
                 ActualRecord$="1"
                 Auxil$=GetRecordConvDB$(PulsarPath$,PulsarFile$,ActualRecord$)
                 POKE$ VARPTR(DatabaseRecord),Auxil$  'assign record readed to User Defined Type (DataBaseRecord)
                 CALL PresentRecord          'present record in screen
                 CALL DisableScreen          'inactivate screen controls (no modify state)

                 MSGBOX "NEW RECORD SAVED IN DATABASE!",%MB_ICONINFORMATION,"PLSAR 3.0 Lite"
              END IF
       END SELECT

    ELSE                        'Update button is selected

       CONTROL GET TEXT hDlg, %id_Update TO ActualStatus$
       ActualStatus$=LCASE$(TRIM$(ActualStatus$))

       SELECT CASE ActualStatus$
          CASE "u&pdate" 'update
              CONTROL SET TEXT hDlg,%id_Update,"SAVE"
              CONTROL SET TEXT hDlg,%id_Add,"CANCEL"
              CALL AddCountries   'initialize our combobox Country List
              CALL EnableScreen   'enable screen to update actual record
              CONTROL SET FOCUS hDlg,%id_Company

          CASE "cancel"
              CONTROL SET TEXT hDlg,%id_Update,"U&pdate"
              CONTROL SET TEXT hDlg,%id_Add,"&Add"

              'read the actual record and present it
              Auxil$=GetRecordConvDB$(PulsarPath$,PulsarFile$,ActualRecord$) 'read the record
              POKE$ VARPTR(DatabaseRecord),Auxil$  'assign record readed to User Defined Type (DataBaseRecord)
              CALL PresentRecord          'present record in screen
              CALL DisableScreen          'inactivate screen controls (no modify state)

          CASE ELSE      'save modify record in database

              'verify that the company field are not empty because
              'the company field is the database index field
              CONTROL GET TEXT hDlg,%id_Company TO CompanyName

              IF LEN(TRIM$(CompanyName)) = 0 THEN
                 BEEP           ' Display error message dialog
                 MSGBOX "You must specify a Company Name!",%MB_ICONERROR,"PLSAR 3.0 Lite"
              ELSE
                 CONTROL SET TEXT hDlg,%id_Add,"&Add"
                 CONTROL SET TEXT hDlg,%id_Update,"U&pdate"

                 CALL AssignControlTextToFields 'get the information screen
                                                'and assign it to DatabaseRecord

                 Temp$=ActualRecord$            'preserve the ActualRecord$ value

                 RecordToSave$=DatabaseRecord   'save the modifications into database
                 Auxil$=ModifyRecordConvDB$(PulsarPath$,PulsarFile$,RecordToSave$,Temp$)
                 IF Auxil$<>"ok" THEN
                    MSGBOX "ERROR MODIFING RECORD IN DATABASE!",%MB_ICONERROR,"PLSAR 3.0 Lite"
                 END IF

                 Temp$=ActualRecord$

                 Auxil$=GetRecordConvDB$(PulsarPath$,PulsarFile$,Temp$)
                 POKE$ VARPTR(DatabaseRecord),Auxil$  'assign record readed to User Defined Type (DataBaseRecord)
                 CALL PresentRecord          'present record in screen
                 CALL DisableScreen          'inactivate screen controls (no modify state)

                 MSGBOX "RECORD MODIFY IN DATABASE!",%MB_ICONINFORMATION,"PLSAR 3.0 Lite"

              END IF

       END SELECT

    END IF

    ' Reset the modified flag
    gIsRecModified = %FALSE

    ' Flag data as modified
    gIsFileModified = %TRUE

END FUNCTION


'------------------------------------------------------------------------------
' Find button callback function
'------------------------------------------------------------------------------
CALLBACK FUNCTION FindCallBack

    STATIC hDlg   AS DWORD
    STATIC IsFind AS LONG
    LOCAL  x      AS LONG

    'Is this dialog already active?
    IF ISTRUE IsFind THEN
        ' Yes, switch focus to the edit control in the Find dialog
        CONTROL SET FOCUS hDlg, %IDOK
        EXIT FUNCTION
    END IF

    ' Create our "Find" dialog as a modeless dialog
    DIALOG NEW CBHNDL, "Find Company Name", 0, 0, 150, 50, %DS_MODALFRAME OR %WS_CAPTION OR _
        %DS_CENTER, %WS_EX_TOPMOST OR %WS_EX_TOOLWINDOW TO hDlg
    IF hDlg = 0 THEN EXIT FUNCTION  ' Error occurred
    DIALOG SET COLOR   hDlg, -1, RGB(222,222,255)
    SetProp hDlg, "Parent", CBHNDL

    CONTROL ADD LABEL, hDlg, -1, "Find &Text:", 1, 7, 20, 28, %SS_RIGHT
    CONTROL SET COLOR  hDlg, -1, %BLUE, -2&

    CONTROL ADD TEXTBOX, hDlg, %id_Find,  gszFindText,  30, 7, 110, 14
    CONTROL ADD BUTTON,  hDlg, %IDOK,     "&Find Next", 30, 30, 40, 14, %BS_DEFAULT
    CONTROL ADD BUTTON,  hDlg, %IDCANCEL, "&Cancel",    80, 30, 40, 14 CALL CancelCallBack

    ' Flag the dialog as active so clicking on the FIND button in the main window won't
    ' trigger a 2nd instance of the Find dialog box
    IsFind = hDlg

    ' Display the dialog
    DIALOG SHOW MODELESS hDlg, CALL FindDialogCallBack

    ' Now loop until the dialog is dismissed
    DO
        ' Allow messages to be dispatched
        DIALOG DOEVENTS
        ' Query the dialog size... if the return value is zero, the dialog was destroyed
        DIALOG GET SIZE hDlg TO x, x
    LOOP WHILE x

    ' Reactivate our parent dialog (in case previous window in the z-order was not our
    ' main dialog)
    DIALOG SHOW STATE CBHNDL, %SW_SHOWNORMAL

    ' Clear active flag
    IsFind = %FALSE

END FUNCTION


'------------------------------------------------------------------------------
' Find Dialog callback function
'------------------------------------------------------------------------------
CALLBACK FUNCTION FindDialogCallback

    LOCAL FindText AS ASCIIZ * 1024
    LOCAL x        AS LONG
    LOCAL TextToSearch$,Auxil$

    SELECT CASE CBMSG

    CASE %WM_DESTROY
        RemoveProp CBHNDL, "Parent"

    CASE %WM_COMMAND
        ' Retrieve the search text if the edit control contents change
        ' We save this in a global variable so we can recreate the Find dialog
        ' with the same search text if it is relaunched again
        IF CBCTLMSG = %EN_UPDATE THEN       ' The find text was changed, save it
            CONTROL GET TEXT CBHNDL, CBCTL TO gszFindText
            ' Strip the string down to the minimum
            gszFindText = TRIM$(gszFindText)
            EXIT FUNCTION
        END IF

        ' Now initiate a search as long as the search text is available
        IF CBCTL = %IDOK AND CBCTLMSG = %BN_CLICKED AND ISTRUE LEN(gszFindText) THEN

            'Start search the company name into database
            TextToSearch$=gszFindText
            Auxil$=SARIndexConDB$(PulsarPath$,PulsarFile$,TextToSearch$)
            IF Auxil$="no" THEN
               'if the record is not find in the search, the SARIndexConDB$
               'packed function return "no".
               'SARIndexConDB$ is the Search record And Return Index from
               'Conventional Database (if record exist) function

               MSGBOX "Not Found", %MB_ICONERROR, $AppTitle
            ELSE
               'if record search exist, the SARIndexConDB$ packed function
               'return into Auxil$ variable the record number

               ActualRecord$=Auxil$  'the find record is the new actual
                                     'record
               Auxil$=GetRecordConvDB$(PulsarPath$,PulsarFile$,ActualRecord$) 'read the record
               POKE$ VARPTR(DatabaseRecord),Auxil$  'assign record readed to User Defined Type (DataBaseRecord)
               CALL PresentRecord          'present record in screen
               CALL DisableScreen          'inactivate screen controls (no modify state)

            END IF

        END IF

    END SELECT

END FUNCTION


'------------------------------------------------------------------------------
' Delete button callback function
'------------------------------------------------------------------------------
CALLBACK FUNCTION DeleteCallBack

    LOCAL Result&,Msg$,Auxil$

    Msg$="Are you sure you wish to delete this record?"
    Result= MSGBOX(Msg$,%MB_YESNO,"PLSAR 3.0 LITE")

    IF Result<>6 THEN EXIT FUNCTION '6=YES / 7=NO

    'the user select delete the actual record
    Auxil$=DeleteRecordConvDB$(PulsarPath$,PulsarFile$,ActualRecord$)

    'get the last record saved into database
    MaximumRecord$=GetQuantityOfRecords$(PulsarPath$,PulsarFile$)

    IF VAL(MaximumRecord$)>0 THEN
       ActualRecord$="1"
       Auxil$=GetRecordConvDB$(PulsarPath$,PulsarFile$,ActualRecord$)
       POKE$ VARPTR(DatabaseRecord),Auxil$  'assign record readed to User Defined Type (DataBaseRecord)
       CALL PresentRecord          'present record in screen
       CALL DisableScreen          'inactivate screen controls (no modify state)
    ELSE
       'if not exist not one record saved into database, the
       'program allow save a new record
       ActualRecord$="0"
       CALL CleanScreen
       CALL EnableScreen
    END IF

END FUNCTION



'------------------------------------------------------------------------------
' "Generic" OK callback function for "messagebox" style DDT dialogs
'------------------------------------------------------------------------------
CALLBACK FUNCTION OkCallBack

    DIALOG END CBHNDL, 1

END FUNCTION


'------------------------------------------------------------------------------
' "Generic" CANCEL callback function for "messagebox" style DDT dialogs
'------------------------------------------------------------------------------
CALLBACK FUNCTION CancelCallBack

    DIALOG END CBHNDL, 0

END FUNCTION


'------------------------------------------------------------------------------
' Update the dialog controls
'------------------------------------------------------------------------------
SUB SetButtonStatus

    ' If any current text is modified, make the ADD & Update buttons active
    IF ISTRUE gIsRecModified THEN
        CONTROL ENABLE  hDlg, %id_Add
        CONTROL ENABLE  hDlg, %id_Update
    ELSE
        CONTROL DISABLE hDlg, %id_Add
        CONTROL DISABLE hDlg, %id_Update
    END IF

    ' Update the Index control text
    CONTROL SET TEXT hDlg, %id_Index, FORMAT$(gCurrentEntry) & " / " & FORMAT$(gMaxEntries)

    ' Do we have any records? is not, disable the Prior/Next buttons, etc
    IF ISFALSE gMaxEntries THEN
        CONTROL DISABLE hDlg, %id_Prior
        CONTROL DISABLE hDlg, %id_Next
        CONTROL DISABLE hDlg, %id_Find
        CONTROL DISABLE hDlg, %id_Delete
        EXIT SUB
    END IF

    ' If there are records, allow the Find/Delete buttons
    CONTROL ENABLE hDlg, %id_Find
    CONTROL ENABLE hDlg, %id_Delete

    ' We have records, so set the Prior button status according to the current index position
    IF gCurrentEntry <= 1 THEN
        CONTROL DISABLE hDlg, %id_Prior
    ELSE
        CONTROL ENABLE  hDlg, %id_Prior
    END IF

    ' Update the Next control button
    IF gCurrentEntry >= gMaxEntries THEN
        CONTROL DISABLE hDlg, %id_Next
    ELSE
        CONTROL ENABLE  hDlg, %id_Next
    END IF

END SUB


'------------------------------------------------------------------------------
' Prefill the ComboBox control with the names of countries
'------------------------------------------------------------------------------
SUB AddCountries

    LOCAL x AS LONG

    ' Clear the combo box list
    COMBOBOX RESET hDlg, %id_Country
    FOR x = 1 TO DATACOUNT
        COMBOBOX ADD hDlg, %id_Country, READ$(x)
    NEXT

    DATA "United States", "Afghanistan", "Albania", "Algeria", "Andorra", "Angola", "Anguilla"
    DATA "Antarctica", "Argentina", "Armenia", "Aruba", "Australia", "Austria"
    DATA "Azerbaijan", "Bahamas", "Bahrain", "Bangladesh", "Barbados", "Belarus"
    DATA "Belgium", "Belize", "Benin", "Bermuda", "Bhutan", "Bolivia", "Botswana"
    DATA "Bouvet Island", "BosniaHerzogovina", "Brazil", "Brunei", "Bulgaria"
    DATA "Burkina Faso", "Cambodia", "Cameroon", "Canada", "Cape Verde"
    DATA "Cayman Islands", "Central African Rep", "Chad", "Chile", "China"
    DATA "Christmas Island", "Colombia", "Comoros", "Congo", "Cook Islands"
    DATA "Costa Rica", "Croatia", "Cuba", "Cyprus", "Czech Republic", "Denmark"
    DATA "Djibouti", "Dominica", "Dominican Republic", "East Timor", "Ecuador"
    DATA "Egypt", "El Salvador", "Equatorial Guinea", "Eritrea", "Estonia"
    DATA "Ethiopia", "Falkland Islands", "Faroe Islands", "Fiji", "Finland"
    DATA "France", "France Metropolitan", "French Guiana", "French Polynesia"
    DATA "French S. Territories", "Gabon", "Gambia", "Georgia", "Germany", "Ghana"
    DATA "Gibraltar", "Greece", "Greenland", "Grenada", "Guadeloupe", "Guam"
    DATA "Guatemala", "Guinea", "Guinea-Bissau", "Guyana", "Haiti", "Honduras"
    DATA "Hong Kong", "Hungary", "Iceland", "IN India", "Indonesia", "Iran", "Iraq"
    DATA "Ireland", "Israel", "Italy", "Ivory Coast", "Jamaica", "Japan", "Jordan"
    DATA "Kazakhstan", "Kenya", "Kiribati", "Korea (North)", "Korea (South)"
    DATA "Kuwait", "Kyrgyzstan", "Laos", "Latvia", "Lebanon", "Lesotho", "Liberia"
    DATA "Libya", "Liechtenstein", "Lithuania", "Luxembourg", "Macau", "Macedonia"
    DATA "Madagascar", "Malawi", "Malaysia", "Maldives", "Mali", "Malta"
    DATA "Marshall Islands", "Martinique", "Mauritania", "Mauritius", "Mayotte"
    DATA "Mexico", "Micronesia", "Miquelon", "Moldova", "Monaco", "Mongolia", "Montserrat"
    DATA "Morocco", "Mozambique", "Myanmar", "Namibia", "Nauru", "Nepal"
    DATA "Netherlands", "Nevis", "New Caledonia", "New Guinea", "New Zealand"
    DATA "Nicaragua", "Niger", "Nigeria", "Niue", "Norfolk Island", "Norway"
    DATA "Oman", "Pakistan", "Palau", "Panama", "Paraguay", "Peru", "Philippines"
    DATA "Pitcairn", "Poland", "Portugal", "Principe", "Puerto Rico", "Qatar"
    DATA "Reunion", "Romania", "Russian Federation", "Rwanda", "Saint Kitts"
    DATA "Saint Lucia", "Samoa", "San Marino", "Sao Tome", "Saudi Arabia", "Senegal"
    DATA "Seychelles", "Sierra Leone", "Singapore", "Slovak Republic", "Slovenia"
    DATA "Solomon Islands", "Somalia", "South Africa", "Spain", "Sri Lanka"
    DATA "St. Helena", "St. Pierre", "Sudan", "Suriname", "Swaziland", "Sweden"
    DATA "Switzerland", "Syria", "Taiwan", "Tajikistan", "Tanzania", "Thailand"
    DATA "Tobago", "Togo", "Tokelau", "Tonga", "Trinidad", "Tunisia", "Turkey"
    DATA "Turkmenistan", "Tuvalu", "Uganda", "Ukraine", "United Arab Emirates"
    DATA "United Kingdom", "United States", "Uruguay", "Uzbekistan", "Vanuatu"
    DATA "Vatican City State", "Venezuela", "Viet Nam", "Virgin Islands (BR)"
    DATA "Virgin Islands (US)", "Western Sahara", "Yemen", "Yugoslavia", "Zaire"
    DATA "Zambia", "Zimbabwe", "Other"

END SUB
'------------------------------------------------------------------------------
FUNCTION GetDriveActual(Disco AS STRING,Directorio AS STRING) AS STRING

    'get the actual drive and path in local or network disk

    'GetDriveActual$    puede ser "ok" o "error"
    'disco$             guarda el disco actual
    'directorio$        guarda el directorio actual
    LOCAL tmp$,ActualDir$,pc$,recurso$,caracter%,zxc%,car$
    LOCAL caracter1%,flagsigue%,tmp1%

    ON ERROR GOTO GetDriveActual_Error

    tmp$=""
    disco$=""
    directorio$=""
    ActualDir$=CURDIR$

    SELECT CASE LEFT$(ActualDir$,2)
       CASE "\\"
          'disco de red
          pc$=""
          recurso$=""
          tmp$=""
          caracter%=0
          'obtiene nombre de pc
          FOR zxc%=3 TO LEN (ActualDir$)
             car$=MID$(ActualDir$,zxc%,1)
             IF car$="\" THEN
                caracter%=zxc%  'guarda posicion de "\"
                pc$=MID$(ActualDir$,3,zxc%-3)
                'zxc%-3= 1 es la posicion actual
                '      = 2 es el tamao de "\\"
             END IF
             IF caracter% THEN EXIT FOR
          NEXT zxc%
          'obtiene nombre del recurso compartido
          caracter1%=0
          flagsigue%=1
          FOR zxc%=(caracter%+1) TO LEN(ActualDir$)
             car$=MID$(ActualDir$,zxc%,1)
             IF car$="\" THEN
                caracter1%=zxc% 'guarda posicion de "\"
                recurso$=MID$(ActualDir$,caracter%+1,zxc%-(1+1+LEN(pc$)+2))
                'zxc%-(1+LEN(pc$)+2)    = 1 es la posicion actual
                '                       = 2 es el tamao de "\\"
                '                       = len(pc$) es el nombre de la pc
                '                       = 1 es la correccin sobre visualizacion
                '                           de datos obtenidos
             END IF
             IF caracter1% THEN EXIT FOR
             IF zxc%=LEN(ActualDir$) THEN
                recurso$=MID$(ActualDir$,caracter%+1,zxc%-(LEN(pc$)+2+1))
                'zxc%-(LEN(pc$)+2)      = 2 es el tamao de "\\"
                '                       = len(pc$) es el nombre de la pc
                '                       = 1 es la posicion actual
                flagsigue%=0
             END IF
          NEXT zxc%
          'arma el nombre completo del disco de red
          disco$="\\"+pc$+"\"+recurso$
          'busca el directorio
          IF flagsigue%=1 THEN
             'caracter1 ha guardado la posicion de "\"
             tmp1%=LEN(ActualDir$)-LEN(disco$)
             directorio$=MID$(ActualDir$,caracter1%,tmp1%)
             'tmp1% es el tamao de disco y recurso dentro de la cadena
          ELSE
             directorio$=""
          END IF

          GetDriveActual$="ok"
          EXIT FUNCTION

       CASE ELSE
          'disco local
          SELECT CASE MID$(ActualDir$,2,1)
             CASE ":"
                GetDriveActual$="ok"
                Disco$=LEFT$(ActualDir$,2)
                Directorio$=MID$(ActualDir$,3,LEN(ActualDir$)-2)
                EXIT FUNCTION
             CASE ELSE
                GetDriveActual$="error"
                Disco$=""
                Directorio$=""
                EXIT FUNCTION
          END SELECT
    END SELECT

    EXIT FUNCTION

GetDriveActual_Error:
    GetDriveActual$="error"
    Disco$=""
    Directorio$=""

END FUNCTION

SUB SaveRecordsInDataBase

   'this subrutine allow save four record into database
   'when the database are created and not contain any data

   LOCAL CAMPO%,CallPulsar$,RecordToSave$,RecordSaved%

   RecordSaved%=0

   FOR CAMPO%=1 TO 60 STEP 15

      RecordSaved%=RecordSaved%+1

      DatabaseRecord.Company   =READ$(CAMPO%)
      DatabaseRecord.LastName  =READ$(CAMPO%+1)
      DatabaseRecord.FirstName =READ$(CAMPO%+2)
      DatabaseRecord.Address1  =READ$(CAMPO%+3)
      DatabaseRecord.Address2  =READ$(CAMPO%+4)
      DatabaseRecord.Address3  =READ$(CAMPO%+5)
      DatabaseRecord.CityName  =READ$(CAMPO%+6)
      DatabaseRecord.StateName =READ$(CAMPO%+7)
      DatabaseRecord.ZipCode   =READ$(CAMPO%+8)
      DatabaseRecord.Country   =READ$(CAMPO%+9)
      DatabaseRecord.Phone     =READ$(CAMPO%+10)
      DatabaseRecord.Fax       =READ$(CAMPO%+11)
      DatabaseRecord.Email     =READ$(CAMPO%+12)
      DatabaseRecord.Url       =READ$(CAMPO%+13)
      DatabaseRecord.Comments  =READ$(CAMPO%+14)

      'assign record to variable for call packed function
      RecordToSave$=DatabaseRecord

      'call Save Record and Index Conventional DataBase packed function
      CallPulsar$=SaveIndexConvDB (PulsarPath$,PulsarFile$,RecordToSave$)

      IF CallPulsar$<>"ok" THEN
         'if error occur, packed function return code and
         'description error in string format "ERROR CODE:xxx....."
         MSGBOX "RECORD "+TRIM$(STR$(RecordSaved%))+" IS NOT SAVED:"+CallPulsar$
         EXIT SUB
      END IF

   NEXT CAMPO%

   MSGBOX " RECORDS ARE SAVED !",,"PLSAR 3.0Lite"
   EXIT SUB

   DATA "PowerBasic Inc.","Edmons","Lance","316 Mid Valley Center","",""
   DATA "Carmel","CA","93923","New Zealand","1-831-639-8000","1-831-639-8008"
   DATA "support@powerbasic.com","http://www.powerbasic.com/"
   DATA "Just another one of those software-engineer types that hang out in the Tech Support Dept..."

   DATA "PowerBasic Inc.","Hanlin","Tom","316 Mid Valley Center","",""
   DATA "Carmel","CA","93923","United States","1-831-639-8000","1-831-639-8008"
   DATA "tom@powerbasic.com","http://www.powerbasic.com/"
   DATA "Engineer extraordinaire!"

   DATA "PowerBasic Inc.","Zale","Bob","316 Mid Valley Center","",""
   DATA "Carmel","CA","93923","United States","1-831-639-8000","1-831-639-8008"
   DATA "bob@powerbasic.com","http://www.powerbasic.com/"
   DATA "The Head Honcho himself!"

   DATA "Hepika Solutions","Ponce","Richard","Mathus Hoyos 2705","",""
   DATA "MENDOZA","MZA","M5533BHY","Argentina","54-0261-4510174","54-0261-4510174"
   DATA "ricardodp@losandesinternet.com.ar","http://www.hepika.com.ar"
   DATA "Hepika Solutions: we are programmers... as you"

END SUB

SUB PresentRecord

    'this routine allow present the actual record in screen

    CONTROL SET TEXT hDlg, %id_Index,TRIM$(ActualRecord$)+"/"+TRIM$(MaximumRecord$)

    CONTROL SET TEXT hDlg, %id_Company,   DatabaseRecord.Company
    CONTROL SET TEXT hDlg, %id_LastName,  DatabaseRecord.LastName
    CONTROL SET TEXT hDlg, %id_FirstName, DatabaseRecord.FirstName
    CONTROL SET TEXT hDlg, %id_Address1,  DatabaseRecord.Address1
    CONTROL SET TEXT hDlg, %id_Address2,  DatabaseRecord.Address2
    CONTROL SET TEXT hDlg, %id_Address3,  DatabaseRecord.Address3
    CONTROL SET TEXT hDlg, %id_CityName,  DatabaseRecord.CityName
    CONTROL SET TEXT hDlg, %id_StateName, DatabaseRecord.StateName
    CONTROL SET TEXT hDlg, %id_ZipCode,   DatabaseRecord.ZipCode
    CONTROL SET TEXT hDlg, %id_Phone,     DatabaseRecord.Phone
    CONTROL SET TEXT hDlg, %id_Fax,       DatabaseRecord.Fax
    CONTROL SET TEXT hDlg, %id_Email,     DatabaseRecord.Email
    CONTROL SET TEXT hDlg, %id_Url,       DatabaseRecord.Url
    CONTROL SET TEXT hDlg, %id_Comments,  DatabaseRecord.Comments

    COMBOBOX RESET hDlg, %id_Country
    COMBOBOX ADD hDlg, %id_Country, DatabaseRecord.Country
    COMBOBOX SELECT hDlg, %id_Country, 1

END SUB

SUB CleanScreen

    'this routine clear the screen and allow input
    'new fields when the user desire enter a new record

    CONTROL SET TEXT hDlg, %id_Index,"   "

    CONTROL SET TEXT hDlg, %id_Company,   ""
    CONTROL SET TEXT hDlg, %id_LastName,  ""
    CONTROL SET TEXT hDlg, %id_FirstName, ""
    CONTROL SET TEXT hDlg, %id_Address1,  ""
    CONTROL SET TEXT hDlg, %id_Address2,  ""
    CONTROL SET TEXT hDlg, %id_Address3,  ""
    CONTROL SET TEXT hDlg, %id_CityName,  ""
    CONTROL SET TEXT hDlg, %id_StateName, ""
    CONTROL SET TEXT hDlg, %id_ZipCode,   ""
    CONTROL SET TEXT hDlg, %id_Country,   ""
    COMBOBOX RESET hDlg, %id_Country
    CONTROL SET TEXT hDlg, %id_Phone,     ""
    CONTROL SET TEXT hDlg, %id_Fax,       ""
    CONTROL SET TEXT hDlg, %id_Email,     ""
    CONTROL SET TEXT hDlg, %id_Url,       ""
    CONTROL SET TEXT hDlg, %id_Comments,  ""

END SUB

SUB DisableScreen

    'when the user read the database record, is not
    'possible modify the fields record, except when
    'select the UPDATE button.

    CONTROL DISABLE hDlg, %id_Company
    CONTROL DISABLE hDlg, %id_LastName
    CONTROL DISABLE hDlg, %id_FirstName
    CONTROL DISABLE hDlg, %id_Address1
    CONTROL DISABLE hDlg, %id_Address2
    CONTROL DISABLE hDlg, %id_Address3
    CONTROL DISABLE hDlg, %id_CityName
    CONTROL DISABLE hDlg, %id_StateName
    CONTROL DISABLE hDlg, %id_ZipCode
    CONTROL DISABLE hDlg, %id_Country
    CONTROL DISABLE hDlg, %id_Country
    CONTROL DISABLE hDlg, %id_Phone
    CONTROL DISABLE hDlg, %id_Fax
    CONTROL DISABLE hDlg, %id_Email
    CONTROL DISABLE hDlg, %id_Url
    CONTROL DISABLE hDlg, %id_Comments

END SUB

SUB EnableScreen

    'when the user select UPDATE or ADD buttons
    'is possible to modify the fields record, saving
    'a new record into database or modifing the
    'actual record

    CONTROL ENABLE hDlg, %id_Company
    CONTROL ENABLE hDlg, %id_LastName
    CONTROL ENABLE hDlg, %id_FirstName
    CONTROL ENABLE hDlg, %id_Address1
    CONTROL ENABLE hDlg, %id_Address2
    CONTROL ENABLE hDlg, %id_Address3
    CONTROL ENABLE hDlg, %id_CityName
    CONTROL ENABLE hDlg, %id_StateName
    CONTROL ENABLE hDlg, %id_ZipCode
    CONTROL ENABLE hDlg, %id_Country
    CONTROL ENABLE hDlg, %id_Phone
    CONTROL ENABLE hDlg, %id_Fax
    CONTROL ENABLE hDlg, %id_Email
    CONTROL ENABLE hDlg, %id_Url
    CONTROL ENABLE hDlg, %id_Comments

END SUB

SUB AssignControlTextToFields

    'when the user add a new record or modify the actual
    'record, this routine allow get the fields and
    'saving its into the DatabaseRecord global variable

    CONTROL GET TEXT hDlg, %id_Company    TO DatabaseRecord.Company
    CONTROL GET TEXT hDlg, %id_LastName   TO DatabaseRecord.LastName
    CONTROL GET TEXT hDlg, %id_FirstName  TO DatabaseRecord.FirstName
    CONTROL GET TEXT hDlg, %id_Address1   TO DatabaseRecord.Address1
    CONTROL GET TEXT hDlg, %id_Address2   TO DatabaseRecord.Address2
    CONTROL GET TEXT hDlg, %id_Address3   TO DatabaseRecord.Address3
    CONTROL GET TEXT hDlg, %id_CityName   TO DatabaseRecord.CityName
    CONTROL GET TEXT hDlg, %id_StateName  TO DatabaseRecord.StateName
    CONTROL GET TEXT hDlg, %id_ZipCode    TO DatabaseRecord.ZipCode
    CONTROL GET TEXT hDlg, %id_Country    TO DatabaseRecord.Country
    CONTROL GET TEXT hDlg, %id_Phone      TO DatabaseRecord.Phone
    CONTROL GET TEXT hDlg, %id_Fax        TO DatabaseRecord.Fax
    CONTROL GET TEXT hDlg, %id_Email      TO DatabaseRecord.Email
    CONTROL GET TEXT hDlg, %id_Url        TO DatabaseRecord.Url
    CONTROL GET TEXT hDlg, %id_Comments   TO DatabaseRecord.Comments

END SUB
