'-------------------------------------------------------------------
' PULSAR30_HOWTO.BAS (c)2002 by Ricardo Daniel Ponce
' Source code in PowerBasic 7.0
'
' November 2002
' Plsar 3.0 (c)2002 by Ricardo Daniel Ponce
'                       rponce@lanet.com.ar
'
' HEPIKA SOLUTIONS
' We are programmers... as you
' www.hepika.com.ar
'
'This file contain methods an examples functions that allow
'operate your databases through plsar 3.0
'-------------------------------------------------------------------

'................................
'Orden de compilacion
#COMPILE EXE
'................................
'Inclusin de libreras, recursos y archivos necesarios para WIN NT
#INCLUDE "WIN32API.INC"
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'PULSAR 3.0 Lite
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'Plsar packed functions
#INCLUDE "PULSAR30LITE.INC"


'Record defined in UDT (User Defined Types) for this demostration.
'This structure is used by NewConventionalDB$ packed function
'in the database creation and others functions during the access
'operations into database

TYPE Fields
    Apellido  AS STRING * 30    'last name field
    Nombres   AS STRING * 40    'first name field
    Email     AS STRING * 20    'e-mail field
    Web       AS STRING * 50    'web field
END TYPE
GLOBAL DatabaseRecord AS FieldS 'Database Record used in this program


'auxiliar subs (view descriptions into subs)

DECLARE SUB SaveTenRecordsInConvDB
DECLARE SUB ViewRecordsInConventionalDB(FilePath$,FileNameDataBase$)

'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

    $REGISTER NONE

    '------------------------------------------------------------------
    'BEGIN DEMO
    '------------------------------------------------------------------
    MSGBOX "BEGIN DEMO",,"PULSAR 3.0 - DATABASE MANAGER"

    'variables used in this program

    PulsarPath$="c:\pulsar"         'the database file is saved in this path
    PulsarFile$="Conventional_DB"   'database filename

    '--------------------------------------------------------------------
    'HOW TO CREATE A NEW CONVENTIONAL DATABASE?
    '
    'NewConventionalDB$ packed function allow create a new database file
    'in your disk. Use the Structure$ variable to define the structure
    'coincident with the UDT:
    'TYPE Fields
    '    Apellido  AS STRING * 30    'last name field
    '    Nombres   AS STRING * 40    'first name field
    '    Email     AS STRING * 20    'e-mail field
    '    Web       AS STRING * 50    'web field
    'END TYPE
    'GLOBAL DatabaseRecord AS FieldS 'Database Record used in this program
    'The IndexField$ variable define the index field for ordering the data
    'into file

    Structure$="APELLIDO:30,NOMBRES:40,E-MAIL:20,WEB:50"
    IndexField$="APELLIDO"
    CallPulsar$=NewConventionalDB$(Structure$,IndexField$,PulsarPath$,PulsarFile$)

    IF CallPulsar$<>"ok" THEN
        'If database file is created, the packed function return "ok".
        'If error occur, function return error code in string
        'format: "ERROR CODE:xxx...." (xxx is error code -see plsar manual-)
        MSGBOX "OPERATION IS NOT COMPLETED: "+CallPulsar$
        EXIT FUNCTION
    ELSE
        'database is created
        MSGBOX "OPERACION OK."
        EXIT FUNCTION
    END IF


    '--------------------------------------------------------------------
    'HOW TO SAVE A RECORD INTO DATABASE AND REORDERING THE INDEX TABLE?
    '
    'This routine save 10 records in database file. See description
    'in SaveTenRecordsInConvDB sub
    '
    CALL SaveTenRecordsInConvDB


    '--------------------------------------------------------------------
    'HOW TO REINDEX YOU CONVENTIONAL DATABASE?

    'ReindexConvDB$ packed function force the file reindexation
    'This function must used after SaveConvDB$ (save a record
    'in database an NOT reindex it)

    CallPulsar$=ReindexConvDB$(PulsarPath$,PulsarFile$)

    IF CallPulsar$<>"ok" THEN
        'If database is reindexed, the packed function return "ok".
        'If error occur, function return error code in string
        'format: "ERROR CODE:xxx...." (xxx is error code -see plsar manual-)
        MSGBOX "OPERATION IS NOT COMPLETED: "+CallPulsar$
        EXIT FUNCTION
    ELSE
        MSGBOX "OPERACION OK."
        EXIT FUNCTION
    END IF


    '--------------------------------------------------------------------
    'HOW TO DELETE A RECORD IN YOU DATABASE?

    'DeleteRecordConvDB$ allow delete a record in you database and
    'reordering automatically the index table

    RecordToDelete$="8"   'record number to delete, this number must not
                          'be greater than the last record saved neither
                          'smaller than 1 (Use the GetQuantityOfRecords$
                          'packed function to know the last record)

    CallPulsar$=DeleteRecordConvDB$(PulsarPath$,PulsarFile$,RecordToDelete$)

    IF CallPulsar$<>"ok" THEN
        'If record is deleted, the packed function return "ok".
        'If error occur, function return error code in string
        'format: "ERROR CODE:xxx...." (xxx is error code -see plsar manual-)
        MSGBOX "OPERATION IS NOT COMPLETED: "+CallPulsar$
        EXIT FUNCTION
    ELSE
        MSGBOX "OPERACION OK."
        EXIT FUNCTION
    END IF


    '--------------------------------------------------------------------
    'HOW TO MODIFY A RECORD SAVED IN YOU DATABASE?

    'The ModifyRecordConvDB$ packed function allow modify the data contain
    'in a record saved in you database. If you modify data contain into
    'index field, this function reordering automatically you database.

    DatabaseRecord.Apellido="Ponce (Modify)"
    DatabaseRecord.Nombres ="Richard D."
    DatabaseRecord.Email   ="richard@lanet.com.ar
    DatabaseRecord.Web     ="www.hepika.com.ar/index.htm"

    RecordToModify$="8"   'record number to modify, this number must not
                          'be greater than the last record saved neither
                          'smaller than 1

    NewRecordToSave$=DatabaseRecord   'assign the UDT to NewRecordToSave$
                                      'variable to call the packed function

    CallPulsar$=ModifyRecordConvDB$(PulsarPath$,PulsarFile$,NewRecordToSave$,RecordToModify$)

    IF CallPulsar$<>"ok" THEN
        'If record is modified, the packed function return "ok".
        'If error occur, function return error code in string
        'format: "ERROR CODE:xxx...." (xxx is error code -see plsar manual-)
        MSGBOX "OPERATION IS NOT COMPLETED: "+CallPulsar$
        EXIT FUNCTION
    ELSE
        MSGBOX "OPERACION OK."
        EXIT FUNCTION
    END IF


    '--------------------------------------------------------------------
    'HOW TO VIEW ALL RECORDS SAVED IN YOU DATABASE?
    '
    'This sub allow view all record saved in you database, one to one
    'See complete description in ViewRecordsInConventionalDB sub

    CALL ViewRecordsInConventionalDB(PulsarPath$,PulsarFile$)


    '--------------------------------------------------------------------
    'HOW TO SAVE A RECORD IN YOU DATABASE (AND REINDEX-IT) ONLY IF NOT EXIST PREVIOUSLY?

    'NOTE: SINEConvDB$ is => Save If Not Exist previously in Convencional DataBase
    '
    'The SINEConvDB$ packed function allow save a record and assure it what this
    'record is not saved prevously. Only save the record if the data contain into
    'index file is not find between the saved records of you database

    DatabaseRecord.Apellido= "Rosas"            'is this record (Rosas) saved?
    DatabaseRecord.Nombres = "Ezequiel Edelmiro"
    DatabaseRecord.Email   = "ezequiel_ponce@rapidq.com.ar
    DatabaseRecord.Web     = "www.ezequiel.com.ar"

    RecordToSave$=DatabaseRecord   'assign the UDT to RecordToSave$
                                   'variable to call the packed function

    CallPulsar$=SINEConvDB$(PulsarPath$,PulsarFile$,RecordToSave$)

    IF CallPulsar$<>"ok" THEN
        'If record is saved, the packed function return "ok".
        'If error occur, function return error code in string
        'format: "ERROR CODE:xxx...." (xxx is error code -see plsar manual-)
        MSGBOX "OPERATION IS NOT COMPLETED: "+CallPulsar$

        'NOTE:If record is not saved because exist in you database, SINEConvDB$
        'function return the 123 code error: "CANNOT SAVE RECORD BECAUSE RECORD
        'EXIST IN DATABASE"

        EXIT FUNCTION
    ELSE
        'record is saved
        MSGBOX "OPERACION OK."
        EXIT FUNCTION
    END IF


    '--------------------------------------------------------------------
    'HOW SAVE A RECORD IN YOU DATABASE AND NOT REINDEX?
    '
    'SaveConvDB$ packed function save a record in you database
    'without file reindexation.
    'Use it only for massive entry data that not require search
    'duplicated record in database.
    'Inmediately end the data entry, call the ReindexConvDB$ packed
    'function to force the file reindexation

    DatabaseRecord.Apellido="BBB - ultimo"
    DatabaseRecord.Nombres ="bebe"
    DatabaseRecord.Email   ="bebe@rapidq.com.ar
    DatabaseRecord.Web     ="www.bebe.com.ar"

    RecordToSave$=DatabaseRecord   'assign the UDT to RecordToSave$
                                   'variable to call the packed function

    CallPulsar$=SaveConvDB$(PulsarPath$,PulsarFile$,RecordToSave$)
    IF CallPulsar$<>"ok" THEN
        'If record is saved, the packed function return "ok".
        'If error occur, function return error code in string
        'format: "ERROR CODE:xxx...." (xxx is error code -see plsar manual-)
        MSGBOX "OPERATION IS NOT COMPLETED: "+CallPulsar$
        EXIT FUNCTION
    ELSE
        'record is saved
        MSGBOX "OPERACION OK."
        EXIT FUNCTION
    END IF


    '--------------------------------------------------------------------
    'HOW TO READ A RECORD DIRECTLY FROM DATA FILE?
    '
    'If for any cause you need manipulate the data file that contain
    'the record obviating plsar intervention, get the initial byte in
    'data file that contain the searched record with the ReturnRecordByte$
    'packed function
    '
    'Warning!: The Plsar record manipulation contemplate the network
    '           permissions access. Through this method you expose the
    '           data at corruption or loss
    '
    'This routine allow get all the saved records in you database directly
    'from the data file, without plsar manager intervention

    RecordsInDB$=GetQuantityOfRecords$(PulsarPath$,PulsarFile$) 'get the
                               'quantity of record saved in you database

    IF LEFT$(RecordsInDB$,5)="ERROR" THEN
        'If error occur, function return error code in string
        'format: "ERROR CODE:xxx...." (xxx is error code -see plsar manual-)
        MSGBOX "OPERATION IS NOT COMPLETED: "+RecordsInDB$
        EXIT FUNCTION
    END IF

    'plsar return the quantity of record saved in you database
    'and contain it into RecordsInDB$ variable in string
    'format

    'this for/next loop allow read records one to one directly
    'from you data file

    FOR Record&=1 TO VAL (RecordsInDB$)

       'the ReturnRecordByte$ packed function return the initial byte
       'in data file that contain the searched record

       ByteInicio$=ReturnRecordByte$(PulsarPath$,PulsarFile$,TRIM$(STR$(Record&)))

       IF LEFT$(ByteInicio$,5)="ERROR" THEN
           'If error occur, function return error code in string
           'format: "ERROR CODE:xxx...." (xxx is error code -see plsar manual-)
           MSGBOX "OPERATION IS NOT COMPLETED: "+RecordsInDB$
           EXIT FOR
       END IF

       'if error not occur, the ByteInicio$ variable contain in
       'string format the byte number

       File$="c:\pulsar\Conventional_DB"+".PLS"  'this is the plsar data file

       'open the plsar data file and read the record
       CLOSE #75
       OPEN File$ FOR BINARY ACCESS READ WRITE LOCK READ WRITE AS #75
       GET #75, VAL(ByteInicio$),DatabaseRecord  'save the record into
       CLOSE #75                                 'DatabaseRecord UTD structure

       'show record readed
       Msg$="RECORD N "+TRIM$(STR$(Record&))+"/ "+TRIM$(RecordsInDB$)+CHR$(10)+CHR$(13)
       Msg$=Msg$+"------------"+CHR$(10)+CHR$(13)
       Msg$=Msg$+DatabaseRecord+CHR$(10)+CHR$(13)
       MSGBOX Msg$

    NEXT Record&


    '--------------------------------------------------------------------
    'HOW TO SEARCH A RECORD IN YOU DATABASE AND GET THE INDEX?
    '
    'NOTE: SARIndexConDB$ is => Search record And Return Index in Conventional DataBase
    '
    'This function allow search a record into you database. If record is find
    'SARIndexConDB$ return the rcord number in string format. If record is not
    'find, return error code.

    DataToSearch$="BBB - ultimo"   'our data to search

    CallPulsar$=SARIndexConDB$(PulsarPath$,PulsarFile$,DataToSearch$)

    IF LEFT$(CallPulsar$,5)="ERROR" THEN
        'If error occur, function return error code in string
        'format: "ERROR CODE:xxx...." (xxx is error code -see plsar manual-)
        MSGBOX "OPERATION IS NOT COMPLETED: "+CallPulsar$
        EXIT FUNCTION
    ELSE
        MSGBOX "RECORD CONTAIN THE DATA: Record number "+CallPulsar$
        EXIT FUNCTION
    END IF


    '--------------------------------------------------------------------
    'HOW TO SEARCH A RECORD IN YOU DATABASE AND GET THE INITIAL BYTE WHAT OCUPE IT
    'INTO THE DATA FILE?
    '
    'NOTE: SARByteConDB$ is => Search record And Return initial Byte in Conventional DataBase
    '
    'This packed function allow search a record in you database and, if
    'record is find, get the initial byte in data file for manipulate it
    'obviating plsar intervention
    '
    'Warning!: The Plsar record manipulation contemplate the network
    '           permissions access. Through this method you expose the
    '           data at corruption or loss

    DataToSearch$="BBB - ultimo"   'our data to search

    CallPulsar$=SARByteConDB$(PulsarPath$,PulsarFile$,DataToSearch$)

    IF LEFT$(CallPulsar$,5)="ERROR" THEN
        'If error occur, function return error code in string
        'format: "ERROR CODE:xxx...." (xxx is error code -see plsar manual-)
        MSGBOX "OPERATION IS NOT COMPLETED: "+CallPulsar$
        EXIT FUNCTION
    END IF

    'If record is find SARByteConDB$ return the initial byte in string format

    File$="c:\pulsar\Conventional_DB"+".PLS"  'this is the plsar data file

    'open the plsar data file and read the record
    CLOSE #75
    OPEN File$ FOR BINARY ACCESS READ WRITE LOCK READ WRITE AS #75
    GET #75, VAL(ByteInicio$),DatabaseRecord  'save the record into
    CLOSE #75                                 'DatabaseRecord UTD structure

    'show record
    Msg$="RECORD:"+CHR$(10)+CHR$(13)
    Msg$=Msg$+"------------"+CHR$(10)+CHR$(13)
    Msg$=Msg$+DatabaseRecord+CHR$(10)+CHR$(13)
    MSGBOX Msg$


    '------------------------------------------------------------------
    MSGBOX "THE END!",,"PULSAR DATABASE MANAGER"

END FUNCTION


'------------------------------------------------------------------
'AUXILIAR SUB'S
'------------------------------------------------------------------

SUB SaveTenRecordsInConvDB

   'This sub show how save records in a program

   LOCAL Record$,CAMPO%

   MSGBOX "This subroutine will save 10 records in a conventional database.",,"PLSAR 3.0"

   FOR CAMPO%=1 TO 50 STEP 5

      'this for/next structure allow read the data and save it
      'into database

      DatabaseRecord.Apellido=READ$(CAMPO%)
      DatabaseRecord.Nombres=READ$(CAMPO%+1)
      DatabaseRecord.Email=READ$(CAMPO%+2)
      DatabaseRecord.Web=READ$(CAMPO%+3)
      Record$=READ$(CAMPO%+4)

      RecordToSave$=DatabaseRecord 'assign the record fields saved in DatabaseRecord UDT
                                   'into RecordToSave$ variable for call packed function

      'SaveIndexConvDB$ packed function allow save a record into you database
      'and reordering the index table automatically

      CallPulsar$=SaveIndexConvDB("c:\pulsar","Conventional_DB",RecordToSave$)

      'if not error occur, function return "ok", if error occur, return
      '"ERROR CODE:XXX.Error_Description" string into CallPulsar$ variable

      IF CallPulsar$<>"ok" THEN
         MSGBOX "OPERATION IS NOT COMPLETED IN RECORD:"+Record$+" "+CallPulsar$
         EXIT SUB
      END IF

   NEXT FIELDS%

   MSGBOX " 10 RECORD SAVED !",,"PLSAR 3.0"
   EXIT SUB

   'this data is saved into you database

   DATA "Rosas","Juan Alberto","ja_rosas@supernet.com.ar","http://www.jar.com.ar","1"
   DATA "Ponce","Ricardo Daniel","ricardodp@yahoo.com","http://www.hepika.com.ar","2"
   DATA "Juarez","Roberto","roberto_juarez@yahoo.com","","3"
   DATA "Negrete","Alberto O.","negrete_alberto@yahoo.com.ar","http://www.negrete.com.ar","4"
   DATA "Monta","Norma","normamonta@sheep.com","http://www.monta_ltda.com","5"
   DATA "Morconi","Luis A.","lamorconi@subte.com.ar","http://www.subte.com.ar/morconi","6"
   DATA "Riquelme","Ariel H.","riquelme@hotmail.com","http://www.riquelme.com.jp","7"
   DATA "Onofre","Dario E.","onofre@ciudad.com.ar","http://www.onofre.com","8"
   DATA "Zalazar","Roberto","rzalazar@yahoo.com","http://www.rulo.com.ar","9"
   DATA "Garcia","Esteban","e_garcia@yahoo.com.ar","http://www.garcia_e.com.ar","10"

END SUB

SUB ViewRecordsInConventionalDB(FilePath$,FileNameDataBase$)

   'this sub allow read all records saved in database

   LOCAL RecordsInDB$,RecordSaved$,Msg$


   'GetQuantityOfRecords$ function read the quantity of records saved
   'in you database

   RecordsInDB$=GetQuantityOfRecords$(FilePath$,FileNameDataBase$)

   IF LCASE$(LEFT$(RecordsInDB$,5))="error" THEN

      'If not error occur, function return the quantity or records saved in
      'you database in string format
      'If error occur, return "ERROR CODE:XXX.Error_Description" string into
      'RecordsInDB$ variable
      MSGBOX "Error reading quantity of records saved: "+RecordsInDB$
      EXIT SUB

   ELSE

      'the RecordsInDB$ variable contain the quantity of record saved in
      'database, the for/next structure allow read one to one

      FOR Reading&=1 TO VAL(RecordsInDB$)

         'the GetRecordConvDB$ function allow read a record in database, the
         'record number must be indicated in string format ( TRIM$(STR$(Reading&))
         'in this case )

         RecordSaved$=GetRecordConvDB$(FilePath$,FileNameDataBase$,TRIM$(STR$(Reading&)))
         IF LCASE$(LEFT$(RecordSaved$,5))="error" THEN

            'If not error occur, function return into the RecordSaved$ variable
            'the record readed.
            'If error occur, return "ERROR CODE:XXX.Error_Description" string into
            'the RecordSaved$ variable

            MSGBOX "Error reading record number:"+TRIM$(STR$(Reading&))
            EXIT SUB

         ELSE

            'error no occur, assign the RecordSaved$ variable data to
            'DatabaseRecord UDT structure
            POKE$ VARPTR(DatabaseRecord),RecordSaved$

            'show the record
            Msg$="RECORD N "+TRIM$(STR$(Reading&))+"/ "+TRIM$(RecordsInDB$)+CHR$(10)+CHR$(13)
            Msg$=Msg$+"  "+CHR$(10)+CHR$(13)
            Msg$=Msg$+"APELLIDO: "+DatabaseRecord.Apellido+CHR$(10)+CHR$(13)
            Msg$=Msg$+"NOMBRES: " +DatabaseRecord.Nombres+CHR$(10)+CHR$(13)
            Msg$=Msg$+"E-MAIL: "  +DatabaseRecord.Email+CHR$(10)+CHR$(13)
            Msg$=Msg$+"WEB: "     +DatabaseRecord.Web

            MSGBOX Msg$

         END IF

      NEXT Reading

   END IF

END SUB
